@@ -1,7 +1,7 @@
 module Tins::Annotate
   def annotate(name)
-    class << self; self; end.class_eval do
-      define_method(name) do |annotation|
+    singleton_class.class_eval do
+      define_method(name) do |annotation = :annotated|
         instance_variable_set "@__annotation_#{name}__", annotation
       end
 
@@ -26,5 +26,13 @@ module Tins::Annotate
         instance_variable_set "@__annotation_#{name}__", nil
       end
     end
+
+    define_method("#{name}_annotations") do
+      self.class.__send__("#{name}_annotations")
+    end
+
+    define_method("#{name}_of") do |method_name|
+      self.class.__send__("#{name}_of", method_name)
+    end
   end
 end
