@@ -11,13 +11,17 @@ module Tins
         def dummy=(value)
           if value.respond_to?(:to_str)
             value = DateTime.parse(value.to_str)
+          elsif value.respond_to?(:to_datetime)
+            value = value.to_datetime
           end
           @dummy = value
         end
 
         def dummy(value = nil)
           if value.nil?
-            @dummy
+            if defined?(@dummy)
+              @dummy
+            end
           else
             begin
               old_dummy = @dummy
