@@ -1,4 +1,3 @@
-require 'tins'
 require 'thread'
 require 'sync'
 
@@ -12,14 +11,10 @@ module Tins
   #
   # The module can be included into other modules/classes to make the methods available.
   module Eigenclass
-    if Object.respond_to?(:singleton_class)
-      alias eigenclass singleton_class
-    else
-      # Returns the eigenclass of this object.
-      def eigenclass
-        class << self; self; end
-      end
+    # Returns the eigenclass of this object.
+    def eigenclass
     end
+    alias eigenclass singleton_class
 
     # Evaluates the _block_ in context of the eigenclass of this object.
     def eigenclass_eval(&block)
@@ -30,8 +25,7 @@ module Tins
   module ClassMethod
     include Eigenclass
 
-    # Define a class method named _name_ using _block_. To be able to take
-    # blocks as arguments in the given _block_ Ruby 1.9 is required.
+    # Define a class method named _name_ using _block_.
     def class_define_method(name, &block)
       eigenclass_eval { define_method(name, &block) }
     end
@@ -57,9 +51,16 @@ module Tins
   module ThreadGlobal
     # Define a thread global variable named _name_ in this module/class. If the
     # value _value_ is given, it is used to initialize the variable.
-    def thread_global(name, default_value = nil)
+    def thread_global(name, default_value = nil, &default)
       is_a?(Module) or raise TypeError, "receiver has to be a Module"
 
+      default_value && default and raise ArgumentError,
+        "require either default_falue or default block"
+
+      if default_value
+        default = -> * { default_value }
+      end
+
       name = name.to_s
       var_name = "@__#{name}_#{__id__.abs}__"
 
@@ -67,14 +68,18 @@ module Tins
       modul = self
 
       define_method(name) do
-        lock.synchronize { modul.instance_variable_get var_name }
+        lock.synchronize {
+          if default && !modul.instance_variable_defined?(var_name)
+            modul.instance_variable_set var_name, default.call
+          end
+          modul.instance_variable_get var_name
+        }
       end
 
       define_method(name + "=") do |value|
         lock.synchronize { modul.instance_variable_set var_name, value }
       end
 
-      modul.instance_variable_set var_name, default_value if default_value
       self
     end
 
@@ -92,55 +97,13 @@ module Tins
   end
 
   module InstanceExec
-    unless (Object.instance_method(:instance_exec) rescue nil)
-      class << self
-        attr_accessor :pool
-        attr_accessor :count
-      end
-      self.count = 0
-      self.pool = []
-
-      # This is a pure ruby implementation of Ruby 1.9's instance_exec method. It
-      # executes _block_ in the context of this object while parsing <i>*args</i> into
-      # the block.
-      def instance_exec(*args, &block)
-        instance = self
-        id = instance_exec_fetch_symbol
-        InstanceExec.module_eval do
-          begin
-            define_method id, block
-            instance.__send__ id, *args
-          ensure
-            remove_method id if method_defined?(id)
-          end
-        end
-      ensure
-        InstanceExec.pool << id
-      end
-
-      private
-
-      @@mutex = Mutex.new
-
-      # Fetch a symbol from a pool in thread save way. If no more symbols are
-      # available create a new one, that will be pushed into the pool later.
-      def instance_exec_fetch_symbol
-        @@mutex.synchronize do
-          if InstanceExec.pool.empty?
-            InstanceExec.count += 1
-            symbol = :"__instance_exec_#{InstanceExec.count}__"
-          else
-            symbol = InstanceExec.pool.shift
-          end
-          return symbol
-        end
-      end
+    def self.included(*)
+      super
+      warn "#{self} is deprecated, but included at #{caller.first[/(.*):/, 1]}"
     end
   end
 
   module Interpreter
-    include InstanceExec
-
     # Interpret the string _source_ as a body of a block, while passing
     # <i>*args</i> into the block.
     #
@@ -255,7 +218,10 @@ module Tins
       variable = "@#{name}"
       define_method(name) do |*args|
         if args.empty?
-          result = instance_variable_get(variable)
+          result =
+            if instance_variable_defined?(variable)
+              instance_variable_get(variable)
+            end
           if result.nil?
             result = if default.empty?
               block && instance_eval(&block)
@@ -281,7 +247,10 @@ module Tins
       variable = "@#{name}"
       define_method(name) do |*args|
         if args.empty?
-          result = instance_variable_get(variable)
+          result =
+            if instance_variable_defined?(variable)
+              instance_variable_get(variable)
+            end
           if result.nil?
             if default.empty?
               block && instance_eval(&block)
@@ -465,6 +434,8 @@ module Tins
   # This module can be included into modules/classes to make the delegate
   # method available.
   module Delegate
+    UNSET = Object.new
+
     # A method to easily delegate methods to an object, stored in an
     # instance variable or returned by a method call.
     #
@@ -478,29 +449,33 @@ module Tins
     #   end
     #
     # _other_method_name_ defaults to method_name, if it wasn't given.
-    def delegate(method_name, obj, other_method_name = method_name)
-      raise ArgumentError, "obj wasn't defined" unless obj
-=begin
-1.9 only:
-      define_method(method_name) do |*args, &block|
-        instance_variable_get(obj).__send__(other_method_name, *args, &block)
-      end
-=end
-      obj = obj.to_s
-      if obj[0] == ?@
-        class_eval <<-EOS
-          def #{method_name}(*args, &block)
-            instance_variable_get('#{obj}').__send__(
-              '#{other_method_name}', *args, &block)
+    #def delegate(method_name, to: UNSET, as: method_name)
+    def delegate(method_name, opts = {})
+      to = opts[:to] || UNSET
+      as = opts[:as] || method_name
+      raise ArgumentError, "to argument wasn't defined" if to == UNSET
+      to = to.to_s
+      case
+      when to[0, 2] == '@@'
+        define_method(as) do |*args, &block|
+          if self.class.class_variable_defined?(to)
+            self.class.class_variable_get(to).__send__(method_name, *args, &block)
           end
-        EOS
+        end
+      when to[0] == ?@
+        define_method(as) do |*args, &block|
+          if instance_variable_defined?(to)
+            instance_variable_get(to).__send__(method_name, *args, &block)
+          end
+        end
+      when (?A..?Z).include?(to[0])
+        define_method(as) do |*args, &block|
+          Tins::DeepConstGet.deep_const_get(to).__send__(method_name, *args, &block)
+        end
       else
-        class_eval <<-EOS
-          def #{method_name}(*args, &block)
-            __send__('#{obj}').__send__(
-              '#{other_method_name}', *args, &block)
-          end
-        EOS
+        define_method(as) do |*args, &block|
+          __send__(to).__send__(method_name, *args, &block)
+        end
       end
     end
   end
@@ -606,7 +581,6 @@ module Tins
     def scope_block(scope_frame, name = :default)
       scope_push(scope_frame, name)
       yield
-      self
     ensure
       scope_pop(name)
     end
